package co.jp.app.service;

import java.util.Collection;
import java.util.Collections;

import org.jetbrains.annotations.NotNull;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import co.jp.app.common.ResultCode;
import co.jp.app.dto.RegistrationDto;
import co.jp.app.entity.UserEntity;
import co.jp.app.exception.BusinessException;
import co.jp.app.repository.UserRepository;

@Service
public class UserService implements UserDetailsService {

	private final UserRepository userRepository;
	private final PasswordEncoder passwordEncoder;

	public UserService(UserRepository userRepository, PasswordEncoder passwordEncoder) {
		this.userRepository = userRepository;
		this.passwordEncoder = passwordEncoder;
	}

	
	public UserEntity registerNewUser(@NotNull RegistrationDto registrationDto) throws BusinessException {

		if (userRepository.existsByName(registrationDto.getName())) {
			throw new BusinessException(ResultCode.USER_USERNAME_ALREADY_EXISTS,"error: Name" + registrationDto.getName() + " had been used");
		}

		//密码最短6位限制
		if (registrationDto.getPassword() == null || registrationDto.getPassword().length() < 6) {
			throw new BusinessException(ResultCode.USER_PASSWORD_TOO_SHORT);
		}

		UserEntity newUser = new UserEntity();
		newUser.setName(registrationDto.getName());
		newUser.setEmail(registrationDto.getEmail());
		newUser.setPassword(passwordEncoder.encode(registrationDto.getPassword()));

		return userRepository.save(newUser);
	}

	
	
	@Override
	@Transactional(readOnly = true)
	public UserDetails loadUserByUsername(String name) throws UsernameNotFoundException {
		UserEntity userEntity = userRepository.findByName(name)
				.orElseThrow(() -> new UsernameNotFoundException(name + " not found"));

		Collection<? extends GrantedAuthority> authorities = Collections
				.singletonList(new SimpleGrantedAuthority("ROLE_USER"));

		return new User(
				userEntity.getName(),
				userEntity.getPassword(),
				true, // enabled
				true, // accountNonExpired
				true, // credentialsNonExpired
				true, // accountNonLocked
				authorities // role
		);
	}
}